extern long xbios();

/* DSP XBIOS functions (trap #14) */
#define	Dsp_DoBlock(a,b,c,d)		(void)	xbios(96,a,b,c,d)
#define	Dsp_BlkHandShake(a,b,c,d)	(void)	xbios(97,a,b,c,d)
#define	Dsp_BlkUnpacked(a,b,c,d)	(void)	xbios(98,a,b,c,d)
#define	Dsp_InStream(a,b,c,d)		(void)	xbios(99,a,b,c,d)
#define	Dsp_OutStream(a,b,c,d)		(void)	xbios(100,a,b,c,d)
#define	Dsp_IOStream(a,b,c,d,e,f)	(void)	xbios(101,a,b,c,d,e,f)
#define	Dsp_RemoveInterrupts(a)		(void)	xbios(102,a)
#define	Dsp_GetWordSize()			(int)	xbios(103)
#define	Dsp_Lock()					(int)	xbios(104)
#define	Dsp_Unlock()				(void)	xbios(105)
#define	Dsp_Available(a,b)			(void)	xbios(106,a,b)
#define	Dsp_Reserve(a,b)			(int)	xbios(107,a,b)
#define	Dsp_LoadProg(a,b,c)			(int)	xbios(108,a,b,c)
#define	Dsp_ExecProg(a,b,c)			(void)	xbios(109,a,b,c)
#define	Dsp_ExecBoot(a,b,c)			(void)	xbios(110,a,b,c)
#define	Dsp_LodToBinary(a,b)		(long)	xbios(111,a,b)
#define	Dsp_TriggerHC(a)			(void)	xbios(112,a)
#define	Dsp_RequestUniqueAbility()	(int)	xbios(113)
#define	Dsp_GetProgAbility()		(int)	xbios(114)
#define	Dsp_FlushSubroutines()		(void)	xbios(115)
#define	Dsp_LoadSubroutine(a,b,c)	(int)	xbios(116,a,b,c)
#define	Dsp_InqSubrAbility(a)		(int)	xbios(117,a)
#define	Dsp_RunSubroutine(a)		(int)	xbios(118,a)
#define	Dsp_Hf0(a)					(int)	xbios(119,a)
#define	Dsp_Hf1(a)					(int)	xbios(120,a)
#define	Dsp_Hf2()					(int)	xbios(121)
#define	Dsp_Hf3()					(int)	xbios(122)
#define	Dsp_BlkWords(a,b,c,d)		(void)	xbios(123,a,b,c,d)
#define	Dsp_BlkBytes(a,b,c,d)		(void)	xbios(124,a,b,c,d)
#define	Dsp_HStat()					(char)	xbios(125)
#define Dsp_SetVectors(a,b)			(void)	xbios(126,a,b)
#define	Dsp_MultBlocks(a,b,c,d)		(void)	xbios(127,a,b,c,d)
